/* libutf8++/src/lib/decoder.h
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

/*! \brief Stateful UTF-8 decoder object.

This object is used for stateful decoding of a UTF-8 byte stream. It can be fed the data in
arbitrary chunks, even split on non-character boundaries. It writes its output into a wide character
string.

A variety of error handling modes are available. The default is to throw a BadUTF8Sequence
exception, but you can change this with skipOnError() or replaceOnError().

*/
class Decoder {
public:
    /*! \brief Constructor.

    \param hint Hint at number of characters to allocate space for in decoder buffer.

    The constructor sets up the UTF-8 decoder. You can provide a hint as to the size of your input
    stream chunks. This hint is the number of characters to allocate in the output buffer. If,
    during a single decode operation, this buffer is filled, then it is doubled in size.

     */
    Decoder(size_t hint = 25);



    /// Destructor.
    ~Decoder();



    /// Result of decoding operations (appended to).
    std::wstring decoded;



    /*! \brief UTF-8 decoder.

    \param str Pointer to source data.
    \param amt Number of bytes in source data (-1 for null terminated strings).
    \throws BadUTF8Sequence.

    This function will decode a chunk of UTF-8 data. The decoded data will be appended to whatever
    is contained in the string decoded. You can check if the decoder ended on a character boundary
    or not by calling complete().

    */
    void decode(const char* str, ssize_t amt);



    /// Decode data stored in a std::string.
    void decode(const std::string& str);



    /// Returns \a true if the last call to \a decode() ended on a character boundary.
    bool complete() const;



    /*! \brief Resets the parser for a new UTF-8 stream.

    This function will clear the internal state of the decoder so that it is ready for data from a
    new source. This can be used if you have opened a new file, accepted a new connection, recovered
    from an error, etc. It will also clear \a decoded.

    */
    void reset();



    /*! \brief Set error handling to \e skip mode.

    This function will set the error handling into \e skip mode. In this mode, any invalid UTF-8
    byte sequences will simply be skipped altogether, and will not have any effect on the output in
    \a decoded.

    */
    void skipOnError();



    /*! \brief Set error handling to \e replace mode.

    \param ch The replacement character that will appear in the output.

    This function will set the error handling into \e replace mode. In this mode, any invalid UTF-8
    byte sequences will be skipped, and a replacement character \a ch will be placed onto the output
    in \a decoded. The default parameter is the unicode replacement character, which should look
    like an upside-down question mark.

    */
    void replaceOnError(wchar_t ch = 0xFFFD);



    /*! \brief Set error handling to \e exception mode (default).

    This function will set the error handling to \e exception mode. In this mode, any invalid
    UTF-8 byte sequences will cause a BadUTF8Sequence exception to be thrown. This is the default
    mode.

    */
    void exceptionOnError();



private:
    struct utf8_decode_state ctx;
    wchar_t replaceChar;

    static enum utf8_decode_error_action _skipOnError
        (const struct utf8_decode_state *ctx, enum utf8_decode_error error, wchar_t *newch);
    static enum utf8_decode_error_action _replaceOnError
        (const struct utf8_decode_state *ctx, enum utf8_decode_error error, wchar_t *newch);
    static enum utf8_decode_error_action _exceptionOnError
        (const struct utf8_decode_state *ctx, enum utf8_decode_error error, wchar_t *newch);
};

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
