/* libutf8++/src/lib/encoder.h
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

/*! \brief UTF-8 encoder object.

This object is used to encode Unicode wide characters into UTF-8. It can be fed chunks of characters
which it then encodes, appending the result to an internal buffer.

*/
class Encoder {
public:
    /*! \brief Constructor.

    \param hint Number of bytes to allocate for the encoding buffer.

    The constructor sets up the encoder and allocates some space for an internal buffer. You can
    hint at how large you expect the chunks to be encoded will be. If an encoding operation fills
    the buffer without consuming all the input data, the buffer will be doubled in size for the
    next round.

    */
    Encoder(size_t hint = 100);

    /// Destructor.
    virtual ~Encoder();



    /// UTF-8 output data is appended to this string.
    std::string encoded;



    /*! \brief Encode some data into UTF-8.

    \param str Pointer to the character array to encode.
    \param amt Number of characters to encode.

    This function performs an encoding of some Unicode characters into UTF-8. It appends the result
    onto \a encoded.

    */
    void encode(const wchar_t* str, ssize_t amt);

    /// Encode a std::wstring.
    void encode(const std::wstring& str);



    /// Reset the encoder for a new character stream.
    void reset();



    /*! \brief Set error handling to \e skip mode.

    This function will set the error handling into \e skip mode. In this mode, any invalid UTF-8
    byte sequences will simply be skipped altogether, and will not have any effect on the output in
    \a decoded.

    */
    void skipOnError();



    /*! \brief Set error handling to \e replace mode.

        \param ch The replacement character that will appear in the output.

    This function will set the error handling into \e replace mode. In this mode, any invalid UTF-8
    byte sequences will be skipped, and a replacement character \a ch will be placed onto the output
    in \a decoded. The default parameter is the unicode replacement character, which should look
    like an upside-down question mark.

    */
    void replaceOnError(wchar_t ch = 0xFFFD);



    /*! \brief Set error handling to \e exception mode (default).

    This function will set the error handling to \e exception mode. In this mode, any invalid
    UTF-8 byte sequences will cause a BadUTF8Sequence exception to be thrown. This is the default
    mode.

    */
    void exceptionOnError();



private:
    struct utf8_encode_state ctx;
    wchar_t replaceChar;

    static enum utf8_encode_error_action _skipOnError
        (const struct utf8_encode_state *ctx, wchar_t *newch);
    static enum utf8_encode_error_action _replaceOnError
        (const struct utf8_encode_state *ctx, wchar_t *newch);
    static enum utf8_encode_error_action _exceptionOnError
        (const struct utf8_encode_state *ctx, wchar_t *newch);
};

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
