/* libutf8++/src/lib/exception.h
 *
 *  (c)2006, Laurence Withers. Released under the GNU GPL. See file
 *  COPYING for more information / terms of license.
*/

/*! \brief Exception base class.

This is the base class for all libutf8 exceptions. It contains one member, \a reason, which allows
you to print a human-readable description of the error. To recover the actual type, you can refer
to the more specific derived classes.

*/
class Error : public std::exception {
public:
    /// Human-readable reason for error.
    std::string reason;

    /// Constructor.
    Error(const std::string& reason);

    /// Destructor.
    virtual ~Error() throw()
    { }

    /// Find what caused the error.
    virtual const char* what();
};



/*! \brief Invalid Unicode character exception.

This exception is thrown when encoding Unicode into UTF-8 and an invalid character is encountered.

*/
class BadUnicodeChar : public Error {
public:
    /// A copy of the invalid character.
    wchar_t badChar;

    /// Line of input data at which error occurred (starts at 1).
    int line;

    /// Column of input data at which error occurred (starts at 1).
    int col;

    /// Character offset of input data at which error occurred.
    int char_offset;

    /// Constructor.
    BadUnicodeChar(const struct utf8_encode_state* ctx);

private:
    static std::string format(const struct utf8_encode_state* ctx);
};



/*! \brief Invalid UTF-8 sequence exception.

This exception is thrown when decoding UTF-8 and an invalid sequence is encountered. This could be
a nonsensical sequence, a redundantly-encounded character or truncated source data. It contains some
variables for allowing detailed diagnostics.

*/
class BadUTF8Sequence : public Error {
public:
    /// Description of the error, for human diagnostics.
    std::string description;

    /// Line of input data at which error occurred (starts at 1).
    int line;

    /// Column of input data at which error occurred (starts at 1).
    int col;

    /// Character offset of input data at which error occurred.
    int char_offset;

    /// Byte offset of input data at which error occurred.
    int byte_offset;

    /// Constructor.
    BadUTF8Sequence(const std::string& description, const struct utf8_decode_state* ctx);

    /// Destructor.
    ~BadUTF8Sequence() throw() { }

private:
    std::string format(const std::string& description, const struct utf8_decode_state* ctx);
};

/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
*/
